//
//  PhidgetLEDPlugIn.m
//  PhidgetLED
//
//  Created by James Sheridan on 26/01/09.
//  Copyright (c) 2009 __MyCompanyName__. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>
#import "PhidgetLEDPlugIn.h"

#define	kQCPlugIn_Name				@"PhidgetLED"
#define	kQCPlugIn_Description		@"Controls a Phidget LED device"

CPhidgetLEDHandle led;

@implementation PhidgetLEDPlugIn
@dynamic inputLEDNumber, inputBrightness;

+ (NSDictionary*) attributes
{
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
    if([key isEqualToString:@"inputLEDNumber"])
        return [NSDictionary dictionaryWithObjectsAndKeys:
                @"LED Number", QCPortAttributeNameKey,
                nil];
	
    if([key isEqualToString:@"inputBrightness"])
        return [NSDictionary dictionaryWithObjectsAndKeys:
                @"Brightness", QCPortAttributeNameKey,
                nil];
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeConsumer;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeNone;
}


int gotAttach(CPhidgetHandle phid, void *context) {
	[(id)context phidgetAdded];
	return 0;
}

int gotDetach(CPhidgetHandle phid, void *context) {
	[(id)context phidgetRemoved];
	return 0;
}

- (id) init
{
	int serial = -1, remote = 0;
	
	if(self = [super init]) {
		/*
		Allocate any permanent resource required by the plug-in.
		*/

		CPhidgetLED_create(&led);
		
		CPhidget_set_OnAttach_Handler((CPhidgetHandle)led, gotAttach, self);
		CPhidget_set_OnDetach_Handler((CPhidgetHandle)led, gotDetach, self);
		
		if(remote)
			CPhidget_openRemote((CPhidgetHandle)led, serial, NULL, NULL);
		else
			CPhidget_open((CPhidgetHandle)led, serial);		
	}
	
	return self;
}

- (void) finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void) dealloc
{
	/*
	Release any resources created in -init.
	*/
	CPhidget_close((CPhidgetHandle)led);
	CPhidget_delete((CPhidgetHandle)led);

	[super dealloc];
}

- (void)phidgetAdded
{
	NSLog(@"Phidget Added");

	int serial, version;
	int numLEDs;
	const char *name;
//	CPhidget_DeviceID devid;
	
	CPhidget_getSerialNumber((CPhidgetHandle)led, &serial);
	CPhidget_getDeviceVersion((CPhidgetHandle)led, &version);
	CPhidget_getDeviceName((CPhidgetHandle)led, &name);
//	CPhidget_getDeviceID((CPhidgetHandle)led, &devid);
	CPhidgetLED_getLEDCount(led,&numLEDs);	
	
	NSLog(@"Serial    = %d", serial);
	NSLog(@"Version Num   = %d", version);
//	NSLog(@"Name      = %@", name);
//	NSLog(@"Device ID = %d", serial);
//	NSLog(@"Num LED's = %d", numLEDs);	
	
	connected = true;
	NSLog(@"Connected");
}

- (void)phidgetRemoved
{
	NSLog(@"Phidget Removed");
	connected = false;
}


@end

@implementation PhidgetLEDPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	*/
	
	LEDNumber  = 0;
	brightness = 0;
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	Called by Quartz Composer whenever the plug-in instance needs to execute.
	Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	
	The OpenGL context for rendering can be accessed and defined for CGL macros using:
	CGLContextObj cgl_ctx = [context CGLContextObj];
	*/

	if (connected)
	{
		if ([self didValueForInputKeyChange:@"inputLEDNumber"]) LEDNumber = self.inputLEDNumber;
		if ([self didValueForInputKeyChange:@"inputBrightness"]) brightness = self.inputBrightness;

		//	double temp = self.inputBrightness;
		NSLog(@"LED number = %f, Brightness = %f", LEDNumber, brightness);

		if(LEDNumber != -1)
			CPhidgetLED_setDiscreteLED(led, LEDNumber, brightness);
	}
	else
	{
		NSLog(@"Phidget not connected");
	}
	
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	*/
}

@end
